-----------------------------------------------------------------------------------------------------------------------------------------
--- BodyQuads V1.0:  Helps converting Body Objects to Quadified Edit Poly
--- Created:  January 6, 2018
--- Modified: January 6, 2018
--- Requirement: Max 2016 or higher

--- AUTHOR:
--- Josef Wienerroither aka "spacefrog"  www.frogsinspace.at | spacefrog@chello.at
-------------------------------------------------------------------------------------------------------------------------------------------

--- HISTORY:
--- Version: V1.0 --- INITIAL RELEASE, DC20180106

--- USAGE:
--- Install macroscript the usual way, drag and drop Macroscript Action onto some toolbar (  "BodyQuad" can be found in Category->Spacefrog Tools )
--- Select Body Objects you want to "quadirfy" and click the button
--- This sets the body Objects tesselation and creates a modifier stack ( including a Edit Poly and meshmooth )
--- There are 2 different mesh resolutions available, the standard ( lowres ) version is triggered simply by button click
--- 
--- Options:
--- BUTTON CLICK : produces low resolution modifier stack on the currently selected Body Objects ( best for lathe surfaces etc... )
--- CTRL + BUTTON CLICK : produces medium resolution modifier stack on the currently selected Body Objects ( best for freeform surfaces )
--- ALT + BUTTON CLICK: removes a previously created stack and resets body objects tesselation settings ( dangerous !!! )


macroScript BodyQuads category:"Spacefrog Tools" buttonText:"BodyQuad Selected" toolTip:"BodyQuad: helps converting Bodyobjects to clean Edit Poly"
(

	function SetDefaultMeshQuality bo=
	(
		if( classof(bo.baseObject) == Body_Object ) do
		(
			bo.LowQualityVDS = off
			bo.MediumQualityVDS = on
			bo.HighQualityVDS = off
			bo.FaceApproxAngleVDS = 20
			bo.EdgeApproxAngleVDS = 0
			bo.FaceChordHeightVDS = 0.2
			bo.EdgeChordHeightVDS = 0
			bo.MaxEdgeLengthPctVDS = 0
		)
	)

	
	function SetLowMeshQuality bo=
	(
		if( classof(bo.baseObject) == Body_Object ) do
		(
			bo.LowQualityVDS = on
			bo.MediumQualityVDS = off
			bo.HighQualityVDS = off
			bo.FaceApproxAngleVDS = 30
			bo.EdgeApproxAngleVDS = 0
			bo.FaceChordHeightVDS = 0
			bo.EdgeChordHeightVDS = 0
			bo.MaxEdgeLengthPctVDS = 0
		)
	)

	
	function SetMedMeshQuality bo=
	(
		if( classof(bo.baseObject) == Body_Object ) do
		(
			bo.LowQualityVDS = on
			bo.MediumQualityVDS = off
			bo.HighQualityVDS = off
			bo.FaceApproxAngleVDS = 20
			bo.EdgeApproxAngleVDS = 0
			bo.FaceChordHeightVDS = 0.1
			bo.EdgeChordHeightVDS = 0
			bo.MaxEdgeLengthPctVDS = 7
		)
	)

	
	function ClearStack bo=
	(
		if( classof(bo.baseObject) == Body_Object AND bo.modifiers.count > 0 ) do
		(
			mods = #( #Meshsmooth, #Edit_Poly, #Quadify_Mesh )
			
			for m in mods do
			(
				thisMod =  bo.modifiers[m]
				if( thisMod != undefined ) do
					deleteModifier bo thisMod
			)
		)
	)
	
	on execute do
	(
		ctrlPressed = keyboard.controlPressed
		altPressed = keyboard.altPressed
		
		actionMan.executeAction 0 "59230"  -- Views: Display Selected with Edged Faces

		if( altPressed ) do			
		(
			for o in selection where ( classof(o.baseObject)==Body_Object ) do
			(
				ClearStack o
				SetDefaultMeshQuality o
				o.DisplayRadioVDS = 0
				o.RenderViewportMeshRA = off
				o.CleanMeshVDS = off

				format "Bodyquads: resetted %\n" o.name
			)
						
			return 0
		)
		
		
		bodyOs = for o in selection where ( classof(o.baseObject)==Body_Object AND 
			(for m in o.modifiers where classof m == Quadify_Mesh collect m).count == 0 ) collect o

			

		
		-- add quadify mod to all selected bodyobjects
		quadMod = Quadify_Mesh()
		quadMod.quadsize=100
		addmodifier bodyOs quadMod	
		meshSmoother = Meshsmooth()

		
		max modify mode
		
		--print bodyOs.count		
			
		if( ctrlPressed	) then
			meshFunc = SetMedMeshQuality
		else
			meshFunc = SetLowMeshQuality

			
			
		for o in bodyOs do
		(
			meshFunc o
			o.DisplayRadioVDS = 1
			o.RenderViewportMeshRA = on
			o.CleanMeshVDS = on
				
			if(	classof(o)==Editable_mesh ) do		-- why is this necessary ?
			(
				addmodifier o (Edit_Poly())
				
				modPanel.setCurrentObject o.modifiers[#Edit_Poly]
					
				PolyToolsModeling.Quadrify false false	-- removes inner edges and should produce mostly quads

				addmodifier o meshSmoother	
			)
		)
	)
)